#!/usr/bin/env python3

# Needs the /usr/bin/time program: apt install time

# Example: ./benchmark.py ~/Devel/bali-phy/git 10 --dir ~/Work --old ~/Devel/bali-phy/old/bali-phy-3.6.0/bin/bali-phy

# See https://llvm.org/docs/Benchmarking.html

import argparse
import re
import subprocess
import sys
import tempfile
import statistics
import os
from git import Repo
import shutil
import logging
from pathlib import Path

def get_times(cmd, cwd, shield=True):
    tmpfile = tempfile.NamedTemporaryFile()

    cmd = ['time' ,'-p','-o', tmpfile.name] + cmd

    if shield:
        cmd = ["cset", "shield", "--exe", "--"] + cmd

    result = subprocess.run(cmd, capture_output=True, cwd=cwd)

    if result.returncode != 0:
        print(result.stderr.decode(),file=sys.stderr)
        return None

    times = dict()
    tmpfile.seek(0)
    for line in tmpfile:
        m = re.match('([^ ]*) +([^ ]*)', line.decode())
        if m:
            times[m.group(1)] = float(m.group(2))

    return times

def benchmark(n, name, cmd, cwd):
    times = []
    for i in range(n):
        print(f"testing {name}: iter {i+1}/{n} ",end='', flush=True)
        time = get_times(cmd+[f'--seed={i}'], cwd)["real"];
        print(f'   {time}s')
        times.append(time)
    return times

def exec_show_result(cmd, **kwargs):
    showcmd = ' '.join([f"'{word}'" for word in cmd])

    subargs = dict()
    if "infile" in kwargs:
        infile = kwargs["infile"]
        subargs["stdin"] = open(infile,encoding='utf-8')
        showcmd += f" < '{infile}'"
    elif "stdin" in kwargs:
        subargs["stdin"] = kwargs["stdin"]

    if "outfile" in kwargs:
        outfile = kwargs["outfile"]
        subargs["stdout"] = open(outfile,'w+',encoding='utf-8')
        showcmd += f" > '{outfile}'"
    elif "stdout" in kwargs:
        subargs["stdout"] = kwargs["stdout"]
    else:
        subargs["stdout"] = subprocess.PIPE

    if "errfile" in kwargs:
        errfile = kwargs["errfile"]
        subargs["stderr"] = open(errfile,'w+',encoding='utf-8')
        showcmd += f" 2> '{errfile}'"
    elif "stderr" in kwargs:
        subargs["stderr"] = kwargs["stderr"]
    else:
        subargs["stderr"] = subprocess.PIPE

    if "cwd" in kwargs:
        subargs["cwd"] = kwargs["cwd"]

    logging.debug(showcmd)
    result = subprocess.run(cmd,**subargs)
    if result.returncode != 0:
        print(f"command: {showcmd}",file=sys.stderr)
        if "outfile" in kwargs and path.exists(kwargs["outfile"]):
            os.remove(kwargs["outfile"])
        if "errfile" in kwargs and path.exists(kwargs["errfile"]):
            os.remove(kwargs["errfile"])
        if "handler" in kwargs:
            handler = kwargs["handler"]
            handler(result.returncode)
    elif "verbose" in kwargs and kwargs["verbose"]:
        print(f"\n\t{showcmd}\n")
    return result

def exec_show(cmd, **kwargs):
    result = exec_show_result(cmd, **kwargs)

    out_message = None
    if "stdout" not in kwargs and "outfile" not in kwargs:
        out_message = result.stdout.decode('utf-8')

    err_message = None
    if "stderr" not in kwargs and "errfile" not in kwargs:
        err_message = result.stderr.decode('utf-8')

    # Always record error messages in the log file.
    if err_message:
        logging.info(f"  err: {err_message}")
    if out_message:
        logging.debug(f"  out: {out_message}")

    code = result.returncode
    if code != 0:
        logging.info(f" exit: {code}")

        logging.info(f" exit: {code}")
        if out_message:
            logging.info(f"  out: {out_message}")
            print(f"  out: {out_message}", file=sys.stdout)
        if err_message:
            logging.info(f"  err: {err_message}")

        if "stop_on_error" in kwargs and kwargs["stop_in_error"]:
            exit(code)
    return (out_message, result)

def configure(source_dir, build_dir, install_parent):

    if not os.path.isdir(build_dir):
        os.mkdir(build_dir)
        meson_out, meson_result = exec_show(['meson','setup', source_dir, build_dir, f'--prefix={install_parent}', '--reconfigure'])
        if meson_result.returncode != 0:
            sys.stderr.write(f"Removing directory {build_dir}!\n")
            logging.warning(f"Removing directory {build_dir}!")
            shutil.rmtree(build_dir)
            return False
        else:
            return True
    else:
        meson_out, meson_result = exec_show(['meson','setup', source_dir, build_dir, f'--prefix={install_parent}', '--reconfigure'])
        if meson_result.returncode != 0:
            return False
        else:
            return True;

def do_compile(build_dir):
    meson_out, meson_result = exec_show(['meson','install', '-C', build_dir])
    return meson_result.returncode == 0

def build(source_dir, build_dir, install_parent):
    if configure(source_dir, build_dir, install_parent):
        logging.info("configure: OK")
        if do_compile(build_dir):
            logging.info("compile: OK")
            return os.path.join(install_parent, "bin", "bali-phy")
        else:
            logging.info("compile: failed")
    else:
        logging.info("configure: failed")
    return None


parser = argparse.ArgumentParser(description="Extract the consensus sequence for a chromosome from a bam file",
                                 epilog= "Example: benchmark.py ERR2679006_dedup_reads.bam PVP01.fasta ERR2679006 --chromosome LT635626 --min-coverage 1 3 5 10 20")



parser.add_argument("repo", help="Path to repo")
parser.add_argument("count", help="Number of commits to benchmark")
parser.add_argument("--dir", default=".", help="Directory to run in")
parser.add_argument("--build", default="build",help="Directory to build in")
parser.add_argument("--install",default="local",help="Directory to install in")
parser.add_argument("--verbose",help="Be verbose",action='store_true')
parser.add_argument("--reps", type=int, default=10, help="Be verbose")

#parser.add_argument("reference", help="Reference file used to map the bam file (.fasta)")
#parser.add_argument("--chromosome", help="The chromosome from which to extract the consensus sequence")
#parser.add_argument("--min-quality", default=25, type=int, help="Quality score required for a base in a read to count (default is 25)")
#parser.add_argument("--min-coverage", default=[2], type=int,nargs='+',help="Number of high quality score reads required to make a call (default is 2)")
#parser.add_argument("--max-coverage",type=int, help="Ignore reference locations with coverage higher than this")
#parser.add_argument("--prefix", help="Prefix chromosome names with this")
#parser.add_argument("--min-mapping-quality", default=1, type=int,nargs='+',help="Skip reads with a lower mapping quality (default is 1)")
#parser.add_argument("--max-soft-clipped", type=int, default=0, help="Skip reads with a higher number of soft-clipped bases (default is 0)")
#parser.add_argument("output", help="Output prefix")
args = parser.parse_args()

if args.verbose:
    logging.basicConfig(level=0)
else:
    logging.basicConfig(level=15)

build_dir = Path(args.build).resolve()
install_dir = Path(args.install).resolve()
run_dir = Path(args.dir)


#1. Find the repo and get the list of commits to analyze
repo = Repo(args.repo)
commits = list(repo.iter_commits(max_count=args.count))
for commit in commits:

    # checkout the commit by running `git checkout {commit}`
    logging.info(f"Checking out commit {commit}")
    repo.git.checkout(commit)

    logging.info(f"Building executable {commit}")
    exe = build(args.repo, build_dir, install_dir)

    cmd = [exe, '48-muscle.fasta', '--pre-burnin=0', '--iter=25']
    #cmd = ['ls','/dev/']
    #cmd = ['ls','/usr/share/doc/']

    print(f"cmd = {cmd}", file=sys.stderr)

    times = benchmark(args.reps, "sha", cmd, run_dir)
    med_time = statistics.median(times)
    sigma_time = statistics.stdev(times)
    print(f"time = {med_time:.2f} +- {sigma_time:.2f}   {commit.message}")





import gdb
import gdb.printing

class expression_ref_printer:
    def __init__(self, val):
        self.val = val

    def to_string(self):
        x = self.val['px']
# Apparently we (Object *) <number> doesn't work, but (reg_var* ) <number> does??
#        return gdb.parse_and_eval('((Object*) %s)->print()' % x)
        return "expression_ref: " + ('%s' % x) # + " " +x.type 

def expression_ref_lookup_function(val):
    if val.type.tag == 'expression_ref':
        return expression_ref_printer(val)
    return None

def register_bali_phy_printers(obj):
    if obj is None:
        obj = gdb
#    obj.pretty_printers.append(expression_ref_lookup_function)

# While I got the infrastructure working, there doesn't seem to be
# an easy way to call the E.px->print() from python.
#
# One suggestion would have bee something like:
#     gdb.parse_and_eval('Object::print(%s)' % x) to call x->print().
#
# From inside gdb, one can run:
#    python print gdb.parse_and_eval('E.px->print()')
# and get
#    "49".
# Apparently this uses the pretty-printer for strings.



#!/usr/bin/env python3
from __future__ import print_function

import os
import subprocess
import re

from collections import defaultdict

def indent(n,s):
    space = ' '*n
    s = space + s
    s = re.sub(r'\n', '\n'+space, s)
    return s

def debug(m):
    sys.stderr.write('DEBUG: ')
    sys.stderr.write(m)
    sys.stderr.write('\n')

def error(m):
    sys.stderr.write('ERROR: ')
    sys.stderr.write(m)
    sys.stderr.write('\n')

def get_precision(x):
    import re
    import math
    numeric_decimal_pattern = r"""
        [-+]? # optional sign
        (?:
            (?: \d* \. (\d+) ) # .1 .12 .123 etc 9.1 etc 98.1 etc
            |
            (?: \d+ \.? ) # 1. 12. 123. etc 1 12 123 etc
        )
        # followed by optional exponent part if desired
        (?: [Ee] [+-]? (\d+) ) ?
     """
    rx = re.compile(numeric_decimal_pattern, re.VERBOSE)
    m = re.match(rx, x)
    if m:
        dec = m.group(1)
        exp = m.group(2)
        if dec:
            dec = len(dec)
        else:
            dec = 0
        if exp:
            exp = int(exp)
        else:
            exp = 0
        prec = exp - dec
        # we need to account for rounding error here
        return math.pow(10,prec)*1.1
    else:
        raise ValueError("'{}' is not a number! Cannot get precision.".format(x))


class Program(object):
    def __init__(self,cmd):
        self.cmd = cmd
        self.name = cmd[0]
        self.exec_file = False
        self.likelihood_regex = r".*likelihood[ \t]*=[ \t]*([^ \t]+).*"
        self.extra_args = []

    def prefix(self):
        return "{}-".format(self.name)

    def control_file(self):
        return "{}command.txt".format(self.prefix())

    def cmdline(self, tester, test_subdir):
        test_dir = tester.dir_for_test(test_subdir)
        args_filename = os.path.join(test_dir,self.control_file())
        if self.exec_file:
            return self.cmd + self.extra_args + [args_filename]
        else:
            args = open(args_filename,'r').read()
            return self.cmd + args.split() + self.extra_args

    def stdin(self, tester, test_subdir):
        return ""

    def read_obtained_likelihood_(self, tester, test_subdir, pat, filename='output'):
        import re
        obtained_output = tester.read_obtained(test_subdir, filename)
        for line in obtained_output.splitlines():
            m = re.match(pat, line)
            if m:
                return m.group(1)
        return None

    def read_obtained_likelihood(self, tester, test_subdir):
        return self.read_obtained_likelihood_(tester, test_subdir, self.likelihood_regex)

class RevBayes(Program):
    def __init__(self, cmd):
        Program.__init__(self,cmd)
        self.name = "revbayes"
        self.exec_file = True
        self.likelihood_regex = r".*likelihood =[ \t]+([^ \t]+)( .*|$)"
        self.extra_args = ['--setOption','outputPrecision=17']

    def control_file(self):
        return "rb-command.Rev"

class Paup(Program):
    def __init__(self, cmd):
        Program.__init__(self,cmd)
        self.name = "paup"
        self.exec_file = True

    def control_file(self):
        return "paup-command.nex"

    def read_obtained_likelihood(self, tester, test_subdir):
        likelihood = Program.read_obtained_likelihood_(self, tester, test_subdir, r"-ln L *([^ \t]+).*")
        if likelihood is not None:
            likelihood = "-"+likelihood
        return likelihood

def splitCmd(cmd):
    for i in range(0,len(cmd)):
        if cmd[i].startswith('-'):
            return (cmd[0:i],cmd[i:])
    return (cmd,[])

class BAliPhy(Program):
    def __init__(self, cmd):
        (cmd1,cmd2) = splitCmd(cmd)
        Program.__init__(self, cmd1)
        self.name = "bali-phy"
        self.likelihood_regex = r".* likelihood = ([^ ]+) .*"
        self.extra_args = cmd2

    def prefix(self):
        return ""

class IQTREE(Program):
    def __init__(self, cmd):
        Program.__init__(self,cmd)
        self.name = "iqtree"
        self.likelihood_regex = r"1. Initial log-likelihood: ([^ ]+)$"
        self.extra_args = ['--show-lh','-redo','-blmin','1.0e-100','-safe']

class raxml_ng(Program):
    def __init__(self, cmd):
        Program.__init__(self,cmd)
        self.name = "raxml-ng"
        self.likelihood_regex = r"Final LogLikelihood: ([^ ]+)$"
        self.extra_args = ['--loglh','--threads','1','--precision','16']


class PhyML(Program):
    def __init__(self, cmd):
        Program.__init__(self,cmd)
        self.name = "phyml"
        self.y = r". Log likelihood of the current tree: ([^ ]+)\$."
        self.likelihood_regex = r". Log likelihood of the current tree: ([^ ]+)\.$"
        self.extra_args = ['--leave_duplicates','-b','0','-o','n','--l_min','1.0e-100']

class hyphymp(Program):
    def __init__(self, cmd):
        Program.__init__(self, cmd)
        self.name = "hyphymp"
        self.likelihood_regex = r"Log Likelihood = ([^ ]+);"
        self.exec_file = True

    def control_file(self):
        return "hyphymp-command.hbl"


class Tester:
    def __init__(self, top_test_dir, data_dir, method):
        self.top_test_dir = top_test_dir
        self.data_dir = data_dir
        self.method = method
        self.NUM_TESTS = 0
        self.FAILED_TESTS = []
        self.XFAILED_TESTS = []

    def dir_for_test(self, test_subdir):
        return os.path.join(self.top_test_dir, test_subdir)

    def rundir_for_test(self, test_subdir):
        return os.path.join(self.top_test_dir, test_subdir)

    def get_test_dirs(self):
        test_dirs = []
        for root, dirs, files in os.walk(top_test_dir):
            if self.method.control_file() in files:
                path = os.path.relpath(root, top_test_dir)
                test_dirs.insert(0,path)
        return test_dirs

    def run_test_cmd(self,test_subdir):
        rundir = self.rundir_for_test(test_subdir)
        prefix = self.method.prefix() + "obtained-"
        obt_outf = os.path.join(rundir, prefix + 'output')
        obt_errf = os.path.join(rundir, prefix + 'error')
        obt_exitf = os.path.join(rundir, prefix +'exit')

        cmd = self.method.cmdline(self, test_subdir)
        stdin = self.method.stdin(self, test_subdir)

        with codecs.open(obt_outf, 'w', encoding='utf-8') as obt_out:
            with codecs.open(obt_errf, 'w', encoding='utf-8') as obt_err:
    #            invocation = '"{}"'.format('" "'.join(cmd))
    #            debug('Running: ' + invocation + ' >"' + obt_outf + '" 2>"' + obt_errf + '" ; echo $? >"' + obt_exitf + '"')
                p = subprocess.Popen(cmd, cwd=rundir, stdin=subprocess.PIPE, stdout=obt_out, stderr=obt_err)
                p.communicate(input=stdin)
                exit_code = p.wait()
                with codecs.open(obt_exitf, 'w', encoding='utf-8') as obt_exit:
                    obt_exit.write('{e:d}\n'.format(e=exit_code))

    def read_expected(self, test_subdir, name):
        test_dir = os.path.join(self.top_test_dir, test_subdir)
        pathname = os.path.join(test_dir, name)
        if not os.path.exists(pathname):
            if name == 'exit':
                return "0"
            else:
                return None
        return codecs.open(pathname, 'r', encoding='utf-8').read().rstrip()

    def read_obtained(self, test_subdir, name):
        rundir = self.rundir_for_test(test_subdir)
        prefix = self.method.prefix()+"obtained-"
        outputf   = os.path.join(rundir, prefix+name)
        return codecs.open(outputf  , 'r', encoding='utf-8').read().rstrip()

    def check_expected(self, test_subdir, name):
        test_dir = self.dir_for_test(test_subdir)
        expected = self.read_expected(test_subdir, name)
        if expected is None:
            return True;
        else:
            obtained = self.read_obtained(test_subdir, name)
            return set(expected.splitlines()).issubset(set(obtained.splitlines()))

    def check_likelihood(self, test_subdir):
        import math

        expected_likelihood = self.read_expected(test_subdir, 'likelihood')

        if not expected_likelihood:
            return None

        obtained_likelihood = self.method.read_obtained_likelihood(self,test_subdir)

        if not obtained_likelihood:
            return "No likelihood found!"

        if expected_likelihood and obtained_likelihood:
            e = float(expected_likelihood);
            o = float(obtained_likelihood);
            diff = o - e

            # expect a relative precision of 1e-14
            prec = abs(e)*1.0e-14
            # but if the likelihood isn't specified that precisely, allow a bigger difference.
            prec = max(prec, get_precision(expected_likelihood))

            if abs(diff) < prec:
                return None
            else:
                rel_diff = abs(diff/e)
                return "likelihood error: absolute={}, relative={} (Got {} but expected {})".format(diff,rel_diff,obtained_likelihood,expected_likelihood)

    def test_xfail(self, test_subdir):
        if os.path.exists(os.path.join(test_subdir, self.method.name, 'xfail')):
            return True;
        return os.path.exists(os.path.join(test_subdir,'xfail'))

    def check_test_output(self,test_subdir):
        test_dir = os.path.join(self.top_test_dir, test_subdir)
        failures = []
        message = ""
        exit_test_failed = False
        if not self.check_expected(test_subdir, 'output'):
            failures.append('output')
        if not self.check_expected(test_subdir, 'error'):
            failures.append('error')
        if not self.check_expected(test_subdir, 'exit'):
            failures.append('exit')
            exit_test_failed = True
        likelihood_message = self.check_likelihood(test_subdir)
        if likelihood_message:
            message += likelihood_message
            failures.append('likelihood')

        if exit_test_failed:
            expected_exit = self.read_expected(test_subdir, 'exit')
            if expected_exit.rstrip() == "0":
                message = self.read_obtained(test_subdir, 'error').rstrip()
                if message == "":
                    message = self.read_obtained(test_subdir, 'output').rstrip()
#                message = "\n".join(message.splitlines()[-6:])
                message = message.lstrip()

        if (len(message) > 0):
            message = indent(5,message)

        xfail = self.test_xfail(test_subdir)

        return (failures,xfail,message)

    def perform_test(self, test_subdir):
        import re
        self.NUM_TESTS += 1

        print("Running {} test:".format(self.method.name),test_subdir," ",end="", flush=True)
        self.run_test_cmd(test_subdir)
        failures,xfail,message = self.check_test_output(test_subdir)
        if not failures:
            print("... ok")
        elif failures:
            if xfail:
                expected="(expected)"
                self.XFAILED_TESTS.append(test_subdir)
            else:
                self.FAILED_TESTS.append(test_subdir)
                expected=""
            print("... FAIL! {} {}".format(failures,expected))
            if message:
                message = message.rstrip('\n')+"\n"
                print(message)

    def test_result_string(self, test_subdir):
        print("Running {} test:".format(self.method.name),test_subdir," ",file=sys.stderr)
        self.run_test_cmd(test_subdir)
        failures,xfail,message = self.check_test_output(test_subdir)
        if xfail and failures:
            return 'XFAIL'
        elif failures:
            return 'FAIL'
        else:
            return 'PASS'

def get_test_method(cmd):
    prog = prog_name(cmd[0])
    if prog == 'wine':
        prog = prog_name(cmd[1]);

    if prog.startswith('bali-phy'):
        return BAliPhy(cmd)
    elif prog == 'rb' or prog.startswith('rb-'):
        return RevBayes(cmd)
    elif prog == 'paup':
        return Paup(cmd)
    elif prog == 'phyml':
        return PhyML(cmd)
    elif prog == 'iqtree':
        return IQTREE(cmd)
    elif prog == 'raxml-ng':
        return raxml_ng(cmd)
    elif prog.startswith('hyphy'):
        return hyphymp(cmd)
    else:
        print("I don't recognize program '{}' - cowardly refusing to run tests for it.".format(prog))
        exit(1)
    
def coverage_dict(top_test_dir, data_dir, progs):
    coverage = defaultdict(dict)
    for prog in progs:
        method = get_test_method([prog])
        tester = Tester(top_test_dir, data_dir, method)
        supported_tests = tester.get_test_dirs()
        for test in supported_tests:
            coverage[test][prog] = "X"
    return coverage

def results_dict(top_test_dir, data_dir, progs):
    results = defaultdict(dict)
    for prog in progs:
        method = get_test_method([prog])
        tester = Tester(top_test_dir, data_dir, method)
        supported_tests = tester.get_test_dirs()
        for test in supported_tests:
            results[test][prog] = tester.test_result_string(test)
    return results

def remove_prefix(s,prefix):
    return s[len(prefix):] if s.startswith(prefix) else s


def test_matrix_from_dict(results,progs):
    header = ["test"]+progs
    rows = [header]
    for test in results:
        testname = test
        testname = remove_prefix(testname, 'tests/')
        testname = remove_prefix(testname, 'likelihood/')
        row = [testname]
        for prog in progs:
            if prog in results[test]:
                row.append(results[test][prog])
            else:
                row.append("")
        rows.append(row)
    return rows


def print_test_matrix(rows):
    for row in rows:
        print(','.join(row))

def prog_name(pathname):
    import re
    filename = os.path.basename(pathname)
    if filename[-4:] == ".exe":
        filename = filename[0:-4]
    return filename

if __name__ == '__main__':
    import codecs
#    import json
    import sys

    script_dir = os.path.split(sys.argv[0])[0]
    script_dir = os.path.abspath(script_dir)

    data_dir = os.path.join(script_dir, 'data')

    top_test_dir = os.getcwd()

    cmd = sys.argv[1:]
    if not cmd:
        print("Please specify which program to test! (e.g. 'bali-phy', 'rb', etc.)")
        exit(1)

    if cmd[0] == 'test':
        cmd = cmd[1:]
    elif cmd[0] == 'coverage':
        progs = cmd[1:]
        print_test_matrix(test_matrix_from_dict(coverage_dict(top_test_dir, data_dir, progs),progs))
        exit(1)
    elif cmd[0] == 'results':
        progs = cmd[1:]
        print_test_matrix(test_matrix_from_dict(results_dict(top_test_dir, data_dir, progs),progs))
        exit(1)

    method = get_test_method(cmd)

    print("Running tests for '{}':\n".format(method.name))
    if os.path.isabs(cmd[0]) and not os.path.exists(cmd[0]):
        print("Executable '{}' not found!".format(cmd[0]))
        exit(1)

    tester = Tester(top_test_dir, data_dir, method)

    for test_subdir in tester.get_test_dirs():
        tester.perform_test(test_subdir)
    if (len(tester.FAILED_TESTS) > 0):
        print("FAIL! ({} unexpected failures, {} expected failures, {} tests total)".format(len(tester.FAILED_TESTS), len(tester.XFAILED_TESTS), tester.NUM_TESTS))
        exit(1)
    else:
        print("SUCCESS! ({} unexpected failures, {} expected failures, {} tests total)".format(len(tester.FAILED_TESTS), len(tester.XFAILED_TESTS), tester.NUM_TESTS))

        exit(0)



#!/usr/bin/python
from __future__ import print_function

def get_alignments(infile):
    alignment = None
    last_sample = None
    with open(infile, 'r') as infilestream:
        for line in infilestream:
            m = re.match("Sample (\d+)\s+Alignment:\s+(.*)$",line)
            if m is not None:
                current_sample = int(m.group(1))
                a = m.group(2)
                # when starting a new group, yield the last alignment and start a new one
                if last_sample != current_sample:
                    if alignment is not None:
                        yield alignment
                    alignment = []
                    last_sample = current_sample
                alignment.append(a)
# Since we only yield an alignment when we start a new one,
# the last one must be handled separately
    yield alignment

def print_alignment(alignment):
    for line in alignment:
        print(line)

if __name__ == '__main__':
    import codecs
    import re
    import sys
    import os

    infile = sys.argv[1]

    num = 0
    for alignment in get_alignments(infile):
        num += 1
        print_alignment(alignment)
        print("\n\n")
    sys.stderr.write("Read {} alignments.\n".format(num))



#!/usr/bin/env python3

import sys
import argparse
import json

def extract_field(j,path):
    for elem in path:
        j = j[elem]
    return j

parser = argparse.ArgumentParser(description="Average piecewise-constant function")
parser.add_argument("filename",type=str,help="File containing JSON log")
parser.add_argument("path",type=str,help="Maximum percentags of Ns allowed")

args = parser.parse_args()

filename = args.filename

path = args.path.split('/')
for i in range(0,len(path)):
    if i < len(path)-1:
        path[i] = path[i]+'/'

with open(filename,'r',encoding='utf8') as file:
    next(file,None)
    for line in file:
        j = json.loads(line)
        print(json.dumps(extract_field(j,path)))





#!/usr/bin/env python3

import sys
import argparse
import json

parser = argparse.ArgumentParser(description="Average piecewise-constant function")
parser.add_argument("--N",type=int,default=100,help="Number of points in reconstruction")
args = parser.parse_args()

N = args.N

def get_piecewise_fn_value(intervals,x):
    for rate,t1,t2 in intervals:
        if t1 <= x <= t2:
            return rate
    return None

def get_time_boundaries(intervals):
    (r0,t1_0,t2_0) = intervals[0]
    t1 = t1_0
    (rn,t1_n,t2_n) = intervals[-1]
    t2 = t2_n
    return (t1,t2)

def count_interval(N, total, intervals):
    t1,t2 = get_time_boundaries(intervals)
    for i in range(0,N):
        t = t1 + i*(t2-t1)/(N-1)
        total[i] += get_piecewise_fn_value(intervals,t)

with sys.stdin as file:
#       with open(filename,'r',encoding='utf8') as file:
    next(file,None)
    n = 0
    total = [0.0 for _ in range(0,N)]
    t1 = None
    t2 = None
    for line in file:
        n += 1
        intervals = json.loads(line)
        if t1 is None:
            t1,t2 = get_time_boundaries(intervals)
        count_interval(N, total, intervals)
    total2 = [ rate / n for rate in total]

    for i in range(0,N):
        t = t1 + i*(t2-t1)/(N-1)
        rate = total2[i]
        print(f"{t}\t{rate}")




#!/usr/bin/env python3

import sys
import argparse
import json
import statistics

# Example:
# ./show-piecewise LiStephens2003Hotspot-7/C1.log.json parameters/rho --output=median > a
# gnuplot
# > plot 'a' with lines

def get_piecewise_fn_value(intervals,x):
    lastRate = None
    for rate,t1,t2 in intervals:
        if t1 <= x <= t2:
            return rate
        else:
            lastRate = rate
    return lastRate

def get_domain_boundaries(intervals):
    (r0,t1_0,t2_0) = intervals[0]
    t1 = t1_0
    (rn,t1_n,t2_n) = intervals[-1]
    t2 = t2_n
    return (t1,t2)

def do_interval(N, values, intervals):
    t1,t2 = get_domain_boundaries(intervals)
    for i in range(0,N):
        t = t1 + i*(t2-t1)/(N-1)
        values[i].append( get_piecewise_fn_value(intervals,t) )

def make_grid(N, intervals):
    t1,t2 = get_domain_boundaries(intervals)
    grid = []
    for i in range(0,N):
        t = t1 + i*(t2-t1)/(N-1)
        grid.append( get_piecewise_fn_value(intervals, t) )
    return grid
        
def extract_field(j,path):
    j2 = j.copy()
    for elem in path:
        j2 = j2[elem]
    return j2

def set_field(j,path,value):
    path2 = path.copy()
    last = path2.pop()
    for elem in path2:
        j = j[elem]
    j[last] = value

def get_path(path_string):
    if not path_string:
        return []

    path = path_string.split('/')
    for i in range(0,len(path)):
        if i < len(path)-1:
            path[i] = path[i]+'/'
    return path

parser = argparse.ArgumentParser(description="Average piecewise-constant function")
parser.add_argument("filename",help="MCON file to analyze")
parser.add_argument("field",help="Field to analyze")
parser.add_argument("--skip",type=int,default=0,help="lines to skip")
parser.add_argument("-N",type=int,default=100,help="Number of points in reconstruction")
parser.add_argument("--output",default="median",help="Output: mean, median, grid")
args = parser.parse_args()

field = get_path(args.field)
N = args.N
skip = args.skip
output = args.output

infile=None
if args.filename == '-':
    infile=sys.stdin
else:
    infile=open(args.filename,"r", encoding='utf8')

# Skip file header    
header = next(infile,None)
if args.output=="grid":
    print(header,end='')

n = 0
values = [[] for _ in range(0,N)]
t1 = None
t2 = None
for line in infile:
    n += 1;
    j = json.loads(line)
    intervals = extract_field(j, field)
    if args.output == "grid":
        intervals = make_grid(N, intervals)
        set_field(j, field, intervals)
        print(json.dumps(j))
    elif n > skip:
        if t1 is None:
            t1,t2 = get_domain_boundaries(intervals)
        do_interval(N, values, intervals)

if args.output == "mean" or args.output == "median":        
    for i in range(0,N):
        t = t1 + i*(t2-t1)/(N-1)
        if  args.output == "mean":
            rate = statistics.mean(values[i])
        elif  args.output == "median":
            rate = statistics.median(values[i])
        print(f"{t}\t{rate}")

